@@ -18,8 +18,7 @@ class RPNHead(RPNTestMixin, AnchorHead):
     """  # noqa: W605
 
     def __init__(self, in_channels, **kwargs):
-        super(RPNHead, self).__init__(
-            1, in_channels, background_label=0, **kwargs)
+        super(RPNHead, self).__init__(1, in_channels, **kwargs)
 
     def _init_layers(self):
         """Initialize layers of the head."""
