@@ -1,6 +1,6 @@
 import torch
+from mmcv.runner import force_fp32
 
-from mmdet.core import force_fp32
 from mmdet.models.builder import ROI_EXTRACTORS
 from .base_roi_extractor import BaseRoIExtractor
 
